@@ -1,6 +1,6 @@
 /**
   Copyright 1993 Bill Triggs <Bill.Triggs@inrialpes.fr>
-  Copyright 1995-2017 Bruno Haible <bruno@clisp.org>
+  Copyright 1995-2019 Bruno Haible <bruno@clisp.org>
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 **/
 /*----------------------------------------------------------------------
   Some test routines for avcall foreign function interface.
@@ -176,6 +176,47 @@ void
   fprintf(out,"->%d\n",ir);
   fflush(out);
 
+  ir = i_i32(i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16,i17,i18,i19,i20,i21,i22,i23,i24,i25,i26,i27,i28,i29,i30,i31,i32);
+  fprintf(out,"->%d\n",ir);
+  fflush(out);
+  ir = 0; clear_traces();
+  av_start_int(a,i_i32,&ir);
+  av_int(a,i1);
+  av_int(a,i2);
+  av_int(a,i3);
+  av_int(a,i4);
+  av_int(a,i5);
+  av_int(a,i6);
+  av_int(a,i7);
+  av_int(a,i8);
+  av_int(a,i9);
+  av_int(a,i10);
+  av_int(a,i11);
+  av_int(a,i12);
+  av_int(a,i13);
+  av_int(a,i14);
+  av_int(a,i15);
+  av_int(a,i16);
+  av_int(a,i17);
+  av_int(a,i18);
+  av_int(a,i19);
+  av_int(a,i20);
+  av_int(a,i21);
+  av_int(a,i22);
+  av_int(a,i23);
+  av_int(a,i24);
+  av_int(a,i25);
+  av_int(a,i26);
+  av_int(a,i27);
+  av_int(a,i28);
+  av_int(a,i29);
+  av_int(a,i30);
+  av_int(a,i31);
+  av_int(a,i32);
+  av_call(a);
+  fprintf(out,"->%d\n",ir);
+  fflush(out);
+
   return;
 }
 void
@@ -1003,6 +1044,122 @@ void
   float fr;
   double dr;
 
+  lr = l_l0J(J1,l9);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+  lr = 0; clear_traces();
+  av_start_long(a,l_l0J,&lr);
+  av_struct(a,J,J1);
+  av_long(a,l9);
+  av_call(a);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+
+  lr = l_l1J(l1,J1,l9);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+  lr = 0; clear_traces();
+  av_start_long(a,l_l1J,&lr);
+  av_long(a,l1);
+  av_struct(a,J,J1);
+  av_long(a,l9);
+  av_call(a);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+
+  lr = l_l2J(l1,l2,J1,l9);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+  lr = 0; clear_traces();
+  av_start_long(a,l_l2J,&lr);
+  av_long(a,l1);
+  av_long(a,l2);
+  av_struct(a,J,J1);
+  av_long(a,l9);
+  av_call(a);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+
+  lr = l_l3J(l1,l2,l3,J1,l9);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+  lr = 0; clear_traces();
+  av_start_long(a,l_l3J,&lr);
+  av_long(a,l1);
+  av_long(a,l2);
+  av_long(a,l3);
+  av_struct(a,J,J1);
+  av_long(a,l9);
+  av_call(a);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+
+  lr = l_l4J(l1,l2,l3,l4,J1,l9);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+  lr = 0; clear_traces();
+  av_start_long(a,l_l4J,&lr);
+  av_long(a,l1);
+  av_long(a,l2);
+  av_long(a,l3);
+  av_long(a,l4);
+  av_struct(a,J,J1);
+  av_long(a,l9);
+  av_call(a);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+
+  lr = l_l5J(l1,l2,l3,l4,l5,J1,l9);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+  lr = 0; clear_traces();
+  av_start_long(a,l_l5J,&lr);
+  av_long(a,l1);
+  av_long(a,l2);
+  av_long(a,l3);
+  av_long(a,l4);
+  av_long(a,l5);
+  av_struct(a,J,J1);
+  av_long(a,l9);
+  av_call(a);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+
+  lr = l_l6J(l1,l2,l3,l4,l5,l6,J1,l9);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+  lr = 0; clear_traces();
+  av_start_long(a,l_l6J,&lr);
+  av_long(a,l1);
+  av_long(a,l2);
+  av_long(a,l3);
+  av_long(a,l4);
+  av_long(a,l5);
+  av_long(a,l6);
+  av_struct(a,J,J1);
+  av_long(a,l9);
+  av_call(a);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+
+  lr = l_l7J(l1,l2,l3,l4,l5,l6,l7,J1,l9);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+  lr = 0; clear_traces();
+  av_start_long(a,l_l7J,&lr);
+  av_long(a,l1);
+  av_long(a,l2);
+  av_long(a,l3);
+  av_long(a,l4);
+  av_long(a,l5);
+  av_long(a,l6);
+  av_long(a,l7);
+  av_struct(a,J,J1);
+  av_long(a,l9);
+  av_call(a);
+  fprintf(out,"->%ld\n",lr);
+  fflush(out);
+
   lr = l_l0K(K1,l9);
   fprintf(out,"->%ld\n",lr);
   fflush(out);
@@ -1350,6 +1507,28 @@ void
   return;
 }
 
+/* Verify that structs larger than 2 words are really
+   passed by value, not accidentally by reference.  */
+void
+  by_value_tests (void)
+{
+  av_alist a;
+  K k;
+
+  k.l1 = l1;
+  k.l2 = l2;
+  k.l3 = l3;
+  k.l4 = l4;
+  fprintf(out,"by_value:%ld,%ld,%ld,%ld\n",k.l1,k.l2,k.l3,k.l4);
+  fflush(out);
+  clear_traces();
+  av_start_void(a,v_clobber_K);
+  av_struct(a,K,k);
+  av_call(a);
+  fprintf(out,"by_value:%ld,%ld,%ld,%ld\n",k.l1,k.l2,k.l3,k.l4);
+  fflush(out);
+}
+
 int
   main (void)
 {
@@ -1363,6 +1542,7 @@ int
   small_structure_return_tests();
   structure_tests();
   gpargs_boundary_tests();
+  by_value_tests();
 
   exit(0);
 }
