@@ -1,7 +1,7 @@
 // Assembly language support for x86_64 CPU.
 // Bruno Haible 2016-12-28
 
-// Copyright (C) 1997-2018 Bruno Haible <bruno@clisp.org>
+// Copyright (C) 1997-2021 Bruno Haible <bruno@clisp.org>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -279,7 +279,25 @@
 // Solaris/ELF
 #define EH_FRAME_SECTION .eh_frame,"aL",link=.text,@unwind
 #else
-// Linux/ELF
+// The eh_frame section was usually writable (flags "aw") in older OS versions,
+// but can be made read-only (flags "a") under specific conditions, depending
+// on the assembler's behaviour (look for EH_TABLES_CAN_BE_READ_ONLY in
+// gcc-11.1.0/gcc/defaults.h).
+// To determine whether the eh_frame section is writable or read-only by
+// default, compile a simple program:
+//   gcc -S -fno-dwarf2-cfi-asm hello.c  (GCC >= 4.4)
+// or
+//   gcc -S hello.c                      (GCC < 4.4)
+// The result is that all modern systems (at least Ubuntu >= 16.04,
+// Red Hat Enterprise Linux >= 5, Fedora >= 13, Alpine Linux >= 3.7,
+// FreeBSD >= 11, DragonFly BSD >= 6, NetBSD >= 7, OpenBSD >= 6)
+// use a read-only eh_frame section.
+#if 1
+// all modern systems
+#define EH_FRAME_SECTION .eh_frame,"a",@progbits
+#else
+// only very old systems
 #define EH_FRAME_SECTION .eh_frame,"aw",@progbits
 #endif
 #endif
+#endif
